@@ -3,6 +3,12 @@
 # Copyright (c) Facebook, Inc. and its affiliates.
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
+
+"""
+.. include:: README.md
+"""
+__docformat__ = "restructuredtext"
+
 from mephisto.operations.registry import fill_registries
 from mephisto.operations.config_handler import init_config
 from mephisto.operations.hydra_config import initialize_named_configs
