@@ -5,6 +5,7 @@
 # LICENSE file in the root directory of this source tree.
 
 import click  # type: ignore
+import os
 from click_default_group import DefaultGroup  # type: ignore
 
 from omegaconf import MISSING
@@ -57,7 +58,11 @@ def config(identifier, value):
 
 
 @cli.command("review")
-@click.argument("review_app_dir", type=click.Path(exists=True))
+@click.argument(
+    "review_app_dir",
+    type=click.Path(exists=True),
+    default=os.path.join(os.path.dirname(__file__), "review/default-ui"),
+)
 @click.option("-p", "--port", type=(int), default=5000)
 @click.option("-o", "--output", type=(str), default="")
 @click.option("-a", "--assets", "assets_dir", type=(str), default=None)
@@ -116,7 +121,7 @@ def review(
 def check():
     """Checks that mephisto is setup correctly"""
     from mephisto.abstractions.databases.local_database import LocalMephistoDB
-    from mephisto.operations.utils import get_mock_requester
+    from mephisto.utils.testing import get_mock_requester
 
     try:
         db = LocalMephistoDB()
@@ -150,7 +155,10 @@ def register_provider(args):
 
     from mephisto.abstractions.databases.local_database import LocalMephistoDB
     from mephisto.operations.registry import get_crowd_provider_from_type
-    from mephisto.operations.utils import parse_arg_dict, get_extra_argument_dicts
+    from mephisto.operations.hydra_config import (
+        parse_arg_dict,
+        get_extra_argument_dicts,
+    )
 
     provider_type, requester_args = args[0], args[1:]
     args_dict = dict(arg.split("=", 1) for arg in requester_args)
@@ -205,7 +213,10 @@ def get_help_arguments(args):
         get_valid_provider_types,
         get_valid_architect_types,
     )
-    from mephisto.operations.utils import get_extra_argument_dicts, get_task_state_dicts
+    from mephisto.operations.hydra_config import (
+        get_extra_argument_dicts,
+        get_task_state_dicts,
+    )
     from textwrap import wrap
 
     VALID_ABSTRACTIONS = ["blueprint", "architect", "requester", "provider", "task"]
@@ -220,9 +231,9 @@ def get_help_arguments(args):
         return
 
     if abstraction == "task":
-        from mephisto.data_model.task_config import TaskConfig
+        from mephisto.data_model.task_run import TaskRun
 
-        target_class = TaskConfig
+        target_class = TaskRun
     else:
         if len(abstraction_equal_split) == 1:
             # querying about the general abstraction
@@ -307,5 +318,50 @@ def get_help_arguments(args):
         click.echo(tabulate(wrap_fields(state_args).values(), headers="keys"))
 
 
+@cli.command("metrics", context_settings={"ignore_unknown_options": True})
+@click.argument("args", nargs=-1)
+def metrics_cli(args):
+    from mephisto.utils.metrics import (
+        launch_servers_and_wait,
+        metrics_are_installed,
+        run_install_script,
+        METRICS_DIR,
+        shutdown_prometheus_server,
+        shutdown_grafana_server,
+    )
+
+    if len(args) == 0 or args[0] not in ["install", "view", "cleanup"]:
+        click.echo(
+            "Usage: mephisto metrics <install|view|cleanup>\n"
+            f"install: Installs Prometheus and Grafana to {METRICS_DIR}\n"
+            f"view: Launches a Prometheus and Grafana server, and shuts down on exit\n"
+            f"cleanup: Shuts down Prometheus and Grafana resources that may have persisted"
+        )
+        return
+    command = args[0]
+    if command == "install":
+        if metrics_are_installed():
+            click.echo(f"Metrics are already installed! See {METRICS_DIR}")
+            return
+        run_install_script()
+    elif command == "view":
+        if not metrics_are_installed():
+            click.echo(
+                f"Metrics aren't installed! Use `mephisto metrics install` first."
+            )
+            return
+        click.echo(f"Servers launching - use ctrl-C to shutdown")
+        launch_servers_and_wait()
+    else:  # command == 'cleanup':
+        if not metrics_are_installed():
+            click.echo(
+                f"Metrics aren't installed! Use `mephisto metrics install` first."
+            )
+            return
+        click.echo(f"Cleaning up existing servers if they exist")
+        shutdown_prometheus_server()
+        shutdown_grafana_server()
+
+
 if __name__ == "__main__":
     cli()
