@@ -10,9 +10,14 @@ from mephisto.data_model.assignment import (
     InitializationData,
     AssignmentState,
 )
-from mephisto.data_model.unit import Unit
+from mephisto.data_model.unit import (
+    Unit,
+    SCREENING_UNIT_INDEX,
+    GOLD_UNIT_INDEX,
+    COMPENSATION_UNIT_INDEX,
+)
 
-from typing import Dict, Optional, List, Any, TYPE_CHECKING, Iterator
+from typing import Dict, Optional, List, Any, TYPE_CHECKING, Iterator, Iterable
 from tqdm import tqdm  # type: ignore
 import os
 import time
@@ -23,16 +28,13 @@ if TYPE_CHECKING:
     from mephisto.abstractions.database import MephistoDB
 
 import threading
-from mephisto.operations.logger_core import get_logger
+from mephisto.utils.logger_core import get_logger
 import types
 
 logger = get_logger(name=__name__)
 
 UNIT_GENERATOR_WAIT_SECONDS = 10
 ASSIGNMENT_GENERATOR_WAIT_SECONDS = 0.5
-SCREENING_UNIT_INDEX = -1
-GOLD_UNIT_INDEX = -2
-COMPENSATION_UNIT_INDEX = -3
 
 
 class GeneratorType(enum.Enum):
@@ -52,7 +54,7 @@ class TaskLauncher:
         self,
         db: "MephistoDB",
         task_run: "TaskRun",
-        assignment_data_iterator: Iterator[InitializationData],
+        assignment_data_iterator: Iterable[InitializationData],
         max_num_concurrent_units: int = 0,
     ):
         """Prepare the task launcher to get it ready to launch the assignments"""
@@ -62,6 +64,7 @@ class TaskLauncher:
         self.assignments: List[Assignment] = []
         self.units: List[Unit] = []
         self.provider_type = task_run.get_provider().PROVIDER_TYPE
+        self.UnitClass = task_run.get_provider().UnitClass
         self.max_num_concurrent_units = max_num_concurrent_units
         self.launched_units: Dict[str, Unit] = {}
         self.unlaunched_units: Dict[str, Unit] = {}
@@ -88,7 +91,7 @@ class TaskLauncher:
     def _create_single_assignment(self, assignment_data) -> None:
         """Create a single assignment in the database using its read assignment_data"""
         task_run = self.task_run
-        task_config = task_run.get_task_config()
+        task_args = task_run.get_task_args()
         assignment_id = self.db.new_assignment(
             task_run.task_id,
             task_run.db_id,
@@ -102,26 +105,20 @@ class TaskLauncher:
         self.assignments.append(assignment)
         unit_count = len(assignment_data.unit_data)
         for unit_idx in range(unit_count):
-            unit_id = self.db.new_unit(
-                task_run.task_id,
-                task_run.db_id,
-                task_run.requester_id,
-                assignment_id,
-                unit_idx,
-                task_config.task_reward,
-                task_run.provider_type,
-                task_run.task_type,
-                task_run.sandbox,
+            unit = self.UnitClass.new(
+                self.db, assignment, unit_idx, task_args.task_reward
             )
-            self.units.append(Unit.get(self.db, unit_id))
+            self.units.append(unit)
             with self.unlaunched_units_access_condition:
-                self.unlaunched_units[unit_id] = Unit.get(self.db, unit_id)
+                self.unlaunched_units[unit.db_id] = unit
 
-    def _try_generating_assignments(self) -> None:
+    def _try_generating_assignments(
+        self, assignment_data_iterator: Iterator[InitializationData]
+    ) -> None:
         """Try to generate more assignments from the assignments_data_iterator"""
         while not self.finished_generators:
             try:
-                data = next(self.assignment_data_iterable)
+                data = next(assignment_data_iterator)
                 self._create_single_assignment(data)
             except StopIteration:
                 self.assignment_thread_done = True
@@ -134,9 +131,12 @@ class TaskLauncher:
             for data in self.assignment_data_iterable:
                 self._create_single_assignment(data)
         else:
+            assert isinstance(
+                self.assignment_data_iterable, types.GeneratorType
+            ), "Must have assignment data generator for this"
             self.assignments_thread = threading.Thread(
                 target=self._try_generating_assignments,
-                args=(),
+                args=(self.assignment_data_iterable,),
                 name="assignment-generator",
             )
             self.assignments_thread.start()
@@ -211,7 +211,7 @@ class TaskLauncher:
             self.launch_url is not None
         ), "Cannot launch an evaluation unit before launching others"
         task_run = self.task_run
-        task_config = task_run.get_task_config()
+        task_args = task_run.get_task_args()
         assignment_id = self.db.new_assignment(
             task_run.task_id,
             task_run.db_id,
@@ -224,18 +224,9 @@ class TaskLauncher:
         assignment = Assignment.get(self.db, assignment_id)
         assignment.write_assignment_data(data)
         self.assignments.append(assignment)
-        unit_id = self.db.new_unit(
-            task_run.task_id,
-            task_run.db_id,
-            task_run.requester_id,
-            assignment_id,
-            unit_type_index,
-            task_config.task_reward,
-            task_run.provider_type,
-            task_run.task_type,
-            task_run.sandbox,
+        evaluation_unit = self.UnitClass.new(
+            self.db, assignment, unit_type_index, task_args.task_reward
         )
-        evaluation_unit = Unit.get(self.db, unit_id)
         evaluation_unit.launch(self.launch_url)
         return evaluation_unit
 
